import java.io.*;
import java.util.*;

public class MyWorld {
   private float viscosity;
   private float g;
   private PrintStream out;
   private Vector<PhysicsElement> elements; // Vector can grow, arrays cannot.

   public MyWorld(PrintStream output){
      g = viscosity= 0.0f;
      out = output;
      elements = new Vector<PhysicsElement>();
   }
   public void setGravity (float gravity) {
      g = gravity;
   }
   public float getGravity () {
      return g;
   }
   public void setViscosity(float v){
      viscosity = v;
   }
   public float getViscosity(){
      return viscosity;
   }
   public void addElement(PhysicsElement e) {
      elements.add(e);
   }
   public PhysicsElement getNear(PhysicsElement anchor) {
      for (PhysicsElement e: elements) 
         if (anchor.collide(e) && (anchor!=e))
            return e;
      return null;      
   }
   private void printSystemHeaders() {
      out.print("Time");
      for (PhysicsElement e: elements)  // print header
         out.print(",\t"+e.getDescription());
      out.println();
   }  
   private void printSystemState(double t) {
      out.print(t);
      for (PhysicsElement e: elements)  
         out.print(",\t" + e.getState());  // print each element state
      out.println();
   }        

   public void simulate ( float delta_t, float endTime, float samplingTime){
      printSystemHeaders();
      for (double t=0, nextStateTime=0; t<endTime; t+=delta_t){
         if (t >= nextStateTime) {       // decide if it is time to print state 
            printSystemState(t);
            nextStateTime += samplingTime;
         } 
         for (PhysicsElement e: elements)  // for each element
            e.computeNextState(delta_t, this); // compute next state
         for (PhysicsElement e: elements)  // for all elements
            e.updateState();                // update its state
      }
   }
} 
